/*
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements.  See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

    package com.ywt.gapi.base.log;

import com.google.protobuf.Message;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.common.stream.StreamObserver;
import org.apache.dubbo.rpc.*;
import org.apache.dubbo.rpc.model.MethodDescriptor;
import org.apache.dubbo.rpc.model.ServiceDescriptor;
import org.apache.dubbo.rpc.model.StubMethodDescriptor;
import org.apache.dubbo.rpc.model.StubServiceDescriptor;
import org.apache.dubbo.rpc.stub.*;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.function.BiConsumer;

public final class DubboYWTLogServiceTriple {

    public static final String SERVICE_NAME = YWTLogService.SERVICE_NAME;

    private static final StubServiceDescriptor serviceDescriptor = new StubServiceDescriptor(SERVICE_NAME,YWTLogService.class);

    static {
        org.apache.dubbo.rpc.protocol.tri.service.SchemaDescriptorRegistry.addSchemaDescriptor(SERVICE_NAME,LogProto.getDescriptor());
        StubSuppliers.addSupplier(SERVICE_NAME, DubboYWTLogServiceTriple::newStub);
        StubSuppliers.addSupplier(YWTLogService.JAVA_SERVICE_NAME,  DubboYWTLogServiceTriple::newStub);
        StubSuppliers.addDescriptor(SERVICE_NAME, serviceDescriptor);
        StubSuppliers.addDescriptor(YWTLogService.JAVA_SERVICE_NAME, serviceDescriptor);
    }

    @SuppressWarnings("all")
    public static YWTLogService newStub(Invoker<?> invoker) {
        return new YWTLogServiceStub((Invoker<YWTLogService>)invoker);
    }

    /**
         * <pre>
         *  日志分类编码，自定义
         * </pre>
         */
    private static final StubMethodDescriptor saveCommentLogMethod = new StubMethodDescriptor("saveCommentLog",
    SaveCommentLogRequest.class, SaveCommentLogResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), SaveCommentLogRequest::parseFrom,
    SaveCommentLogResponse::parseFrom);

    private static final StubMethodDescriptor saveCommentLogAsyncMethod = new StubMethodDescriptor("saveCommentLog",
    SaveCommentLogRequest.class, CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), SaveCommentLogRequest::parseFrom,
    SaveCommentLogResponse::parseFrom);

    private static final StubMethodDescriptor saveCommentLogProxyAsyncMethod = new StubMethodDescriptor("saveCommentLogAsync",
    SaveCommentLogRequest.class, SaveCommentLogResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), SaveCommentLogRequest::parseFrom,
    SaveCommentLogResponse::parseFrom);

    /**
         * <pre>
         *  日志分类描述，自定义
         * </pre>
         */
    private static final StubMethodDescriptor saveOpenHisPushApiLogMethod = new StubMethodDescriptor("saveOpenHisPushApiLog",
    SaveOpenHisPushApiLogRequest.class, SaveOpenHisPushApiLogResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), SaveOpenHisPushApiLogRequest::parseFrom,
    SaveOpenHisPushApiLogResponse::parseFrom);

    private static final StubMethodDescriptor saveOpenHisPushApiLogAsyncMethod = new StubMethodDescriptor("saveOpenHisPushApiLog",
    SaveOpenHisPushApiLogRequest.class, CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), SaveOpenHisPushApiLogRequest::parseFrom,
    SaveOpenHisPushApiLogResponse::parseFrom);

    private static final StubMethodDescriptor saveOpenHisPushApiLogProxyAsyncMethod = new StubMethodDescriptor("saveOpenHisPushApiLogAsync",
    SaveOpenHisPushApiLogRequest.class, SaveOpenHisPushApiLogResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), SaveOpenHisPushApiLogRequest::parseFrom,
    SaveOpenHisPushApiLogResponse::parseFrom);





    public static class YWTLogServiceStub implements YWTLogService{
        private final Invoker<YWTLogService> invoker;

        public YWTLogServiceStub(Invoker<YWTLogService> invoker) {
            this.invoker = invoker;
        }

            /**
         * <pre>
         *  日志分类编码，自定义
         * </pre>
         */
        @Override
        public SaveCommentLogResponse saveCommentLog(SaveCommentLogRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveCommentLogMethod, request);
        }

        public CompletableFuture<SaveCommentLogResponse> saveCommentLogAsync(SaveCommentLogRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveCommentLogAsyncMethod, request);
        }

            /**
         * <pre>
         *  日志分类编码，自定义
         * </pre>
         */
        @Override
        public void saveCommentLog(SaveCommentLogRequest request, StreamObserver<SaveCommentLogResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, saveCommentLogMethod , request, responseObserver);
        }
            /**
         * <pre>
         *  日志分类描述，自定义
         * </pre>
         */
        @Override
        public SaveOpenHisPushApiLogResponse saveOpenHisPushApiLog(SaveOpenHisPushApiLogRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveOpenHisPushApiLogMethod, request);
        }

        public CompletableFuture<SaveOpenHisPushApiLogResponse> saveOpenHisPushApiLogAsync(SaveOpenHisPushApiLogRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveOpenHisPushApiLogAsyncMethod, request);
        }

            /**
         * <pre>
         *  日志分类描述，自定义
         * </pre>
         */
        @Override
        public void saveOpenHisPushApiLog(SaveOpenHisPushApiLogRequest request, StreamObserver<SaveOpenHisPushApiLogResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, saveOpenHisPushApiLogMethod , request, responseObserver);
        }



    }

    public static abstract class YWTLogServiceImplBase implements YWTLogService, ServerService<YWTLogService> {

        private <T, R> BiConsumer<T, StreamObserver<R>> syncToAsync(java.util.function.Function<T, R> syncFun) {
            return new BiConsumer<T, StreamObserver<R>>() {
                @Override
                public void accept(T t, StreamObserver<R> observer) {
                    try {
                        R ret = syncFun.apply(t);
                        observer.onNext(ret);
                        observer.onCompleted();
                    } catch (Throwable e) {
                        observer.onError(e);
                    }
                }
            };
        }

        @Override
        public final Invoker<YWTLogService> getInvoker(URL url) {
            PathResolver pathResolver = url.getOrDefaultFrameworkModel()
            .getExtensionLoader(PathResolver.class)
            .getDefaultExtension();
            Map<String,StubMethodHandler<?, ?>> handlers = new HashMap<>();

            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveCommentLog" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveCommentLogAsync" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveOpenHisPushApiLog" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveOpenHisPushApiLogAsync" );

            BiConsumer<SaveCommentLogRequest, StreamObserver<SaveCommentLogResponse>> saveCommentLogFunc = this::saveCommentLog;
            handlers.put(saveCommentLogMethod.getMethodName(), new UnaryStubMethodHandler<>(saveCommentLogFunc));
            BiConsumer<SaveCommentLogRequest, StreamObserver<SaveCommentLogResponse>> saveCommentLogAsyncFunc = syncToAsync(this::saveCommentLog);
            handlers.put(saveCommentLogProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(saveCommentLogAsyncFunc));
            BiConsumer<SaveOpenHisPushApiLogRequest, StreamObserver<SaveOpenHisPushApiLogResponse>> saveOpenHisPushApiLogFunc = this::saveOpenHisPushApiLog;
            handlers.put(saveOpenHisPushApiLogMethod.getMethodName(), new UnaryStubMethodHandler<>(saveOpenHisPushApiLogFunc));
            BiConsumer<SaveOpenHisPushApiLogRequest, StreamObserver<SaveOpenHisPushApiLogResponse>> saveOpenHisPushApiLogAsyncFunc = syncToAsync(this::saveOpenHisPushApiLog);
            handlers.put(saveOpenHisPushApiLogProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(saveOpenHisPushApiLogAsyncFunc));




            return new StubInvoker<>(this, url, YWTLogService.class, handlers);
        }


        @Override
        public SaveCommentLogResponse saveCommentLog(SaveCommentLogRequest request){
            throw unimplementedMethodException(saveCommentLogMethod);
        }

        @Override
        public SaveOpenHisPushApiLogResponse saveOpenHisPushApiLog(SaveOpenHisPushApiLogRequest request){
            throw unimplementedMethodException(saveOpenHisPushApiLogMethod);
        }





        @Override
        public final ServiceDescriptor getServiceDescriptor() {
            return serviceDescriptor;
        }
        private RpcException unimplementedMethodException(StubMethodDescriptor methodDescriptor) {
            return TriRpcStatus.UNIMPLEMENTED.withDescription(String.format("Method %s is unimplemented",
                "/" + serviceDescriptor.getInterfaceName() + "/" + methodDescriptor.getMethodName())).asException();
        }
    }

}
