/*
* Licensed to the Apache Software Foundation (ASF) under one or more
* contributor license agreements.  See the NOTICE file distributed with
* this work for additional information regarding copyright ownership.
* The ASF licenses this file to You under the Apache License, Version 2.0
* (the "License"); you may not use this file except in compliance with
* the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

    package com.ywt.gapi.drug;

import com.google.protobuf.Message;
import org.apache.dubbo.common.URL;
import org.apache.dubbo.common.stream.StreamObserver;
import org.apache.dubbo.rpc.*;
import org.apache.dubbo.rpc.model.MethodDescriptor;
import org.apache.dubbo.rpc.model.ServiceDescriptor;
import org.apache.dubbo.rpc.model.StubMethodDescriptor;
import org.apache.dubbo.rpc.model.StubServiceDescriptor;
import org.apache.dubbo.rpc.stub.*;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.function.BiConsumer;

public final class DubboDrugServiceTriple {

    public static final String SERVICE_NAME = DrugService.SERVICE_NAME;

    private static final StubServiceDescriptor serviceDescriptor = new StubServiceDescriptor(SERVICE_NAME,DrugService.class);

    static {
        org.apache.dubbo.rpc.protocol.tri.service.SchemaDescriptorRegistry.addSchemaDescriptor(SERVICE_NAME,DrugProto.getDescriptor());
        StubSuppliers.addSupplier(SERVICE_NAME, DubboDrugServiceTriple::newStub);
        StubSuppliers.addSupplier(DrugService.JAVA_SERVICE_NAME,  DubboDrugServiceTriple::newStub);
        StubSuppliers.addDescriptor(SERVICE_NAME, serviceDescriptor);
        StubSuppliers.addDescriptor(DrugService.JAVA_SERVICE_NAME, serviceDescriptor);
    }

    @SuppressWarnings("all")
    public static DrugService newStub(Invoker<?> invoker) {
        return new DrugServiceStub((Invoker<DrugService>)invoker);
    }

    /**
         * <pre>
         * 处方药品信息
         * </pre>
         */
    private static final StubMethodDescriptor saveHisOpenApiPrescriptionMethod = new StubMethodDescriptor("saveHisOpenApiPrescription",
    SaveHisOpenApiPrescriptionRequest.class, SaveHisOpenApiPrescriptionResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), SaveHisOpenApiPrescriptionRequest::parseFrom,
    SaveHisOpenApiPrescriptionResponse::parseFrom);

    private static final StubMethodDescriptor saveHisOpenApiPrescriptionAsyncMethod = new StubMethodDescriptor("saveHisOpenApiPrescription",
    SaveHisOpenApiPrescriptionRequest.class, CompletableFuture.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), SaveHisOpenApiPrescriptionRequest::parseFrom,
    SaveHisOpenApiPrescriptionResponse::parseFrom);

    private static final StubMethodDescriptor saveHisOpenApiPrescriptionProxyAsyncMethod = new StubMethodDescriptor("saveHisOpenApiPrescriptionAsync",
    SaveHisOpenApiPrescriptionRequest.class, SaveHisOpenApiPrescriptionResponse.class, serviceDescriptor, MethodDescriptor.RpcType.UNARY,
    obj -> ((Message) obj).toByteArray(), obj -> ((Message) obj).toByteArray(), SaveHisOpenApiPrescriptionRequest::parseFrom,
    SaveHisOpenApiPrescriptionResponse::parseFrom);





    public static class DrugServiceStub implements DrugService{
        private final Invoker<DrugService> invoker;

        public DrugServiceStub(Invoker<DrugService> invoker) {
            this.invoker = invoker;
        }

            /**
         * <pre>
         * 处方药品信息
         * </pre>
         */
        @Override
        public SaveHisOpenApiPrescriptionResponse saveHisOpenApiPrescription(SaveHisOpenApiPrescriptionRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveHisOpenApiPrescriptionMethod, request);
        }

        public CompletableFuture<SaveHisOpenApiPrescriptionResponse> saveHisOpenApiPrescriptionAsync(SaveHisOpenApiPrescriptionRequest request){
            return StubInvocationUtil.unaryCall(invoker, saveHisOpenApiPrescriptionAsyncMethod, request);
        }

            /**
         * <pre>
         * 处方药品信息
         * </pre>
         */
        @Override
        public void saveHisOpenApiPrescription(SaveHisOpenApiPrescriptionRequest request, StreamObserver<SaveHisOpenApiPrescriptionResponse> responseObserver){
            StubInvocationUtil.unaryCall(invoker, saveHisOpenApiPrescriptionMethod , request, responseObserver);
        }



    }

    public static abstract class DrugServiceImplBase implements DrugService, ServerService<DrugService> {

        private <T, R> BiConsumer<T, StreamObserver<R>> syncToAsync(java.util.function.Function<T, R> syncFun) {
            return new BiConsumer<T, StreamObserver<R>>() {
                @Override
                public void accept(T t, StreamObserver<R> observer) {
                    try {
                        R ret = syncFun.apply(t);
                        observer.onNext(ret);
                        observer.onCompleted();
                    } catch (Throwable e) {
                        observer.onError(e);
                    }
                }
            };
        }

        @Override
        public final Invoker<DrugService> getInvoker(URL url) {
            PathResolver pathResolver = url.getOrDefaultFrameworkModel()
            .getExtensionLoader(PathResolver.class)
            .getDefaultExtension();
            Map<String,StubMethodHandler<?, ?>> handlers = new HashMap<>();

            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveHisOpenApiPrescription" );
            pathResolver.addNativeStub( "/" + SERVICE_NAME + "/saveHisOpenApiPrescriptionAsync" );

            BiConsumer<SaveHisOpenApiPrescriptionRequest, StreamObserver<SaveHisOpenApiPrescriptionResponse>> saveHisOpenApiPrescriptionFunc = this::saveHisOpenApiPrescription;
            handlers.put(saveHisOpenApiPrescriptionMethod.getMethodName(), new UnaryStubMethodHandler<>(saveHisOpenApiPrescriptionFunc));
            BiConsumer<SaveHisOpenApiPrescriptionRequest, StreamObserver<SaveHisOpenApiPrescriptionResponse>> saveHisOpenApiPrescriptionAsyncFunc = syncToAsync(this::saveHisOpenApiPrescription);
            handlers.put(saveHisOpenApiPrescriptionProxyAsyncMethod.getMethodName(), new UnaryStubMethodHandler<>(saveHisOpenApiPrescriptionAsyncFunc));




            return new StubInvoker<>(this, url, DrugService.class, handlers);
        }


        @Override
        public SaveHisOpenApiPrescriptionResponse saveHisOpenApiPrescription(SaveHisOpenApiPrescriptionRequest request){
            throw unimplementedMethodException(saveHisOpenApiPrescriptionMethod);
        }





        @Override
        public final ServiceDescriptor getServiceDescriptor() {
            return serviceDescriptor;
        }
        private RpcException unimplementedMethodException(StubMethodDescriptor methodDescriptor) {
            return TriRpcStatus.UNIMPLEMENTED.withDescription(String.format("Method %s is unimplemented",
                "/" + serviceDescriptor.getInterfaceName() + "/" + methodDescriptor.getMethodName())).asException();
        }
    }

}
